//
//  YHResignDocumentDetailViewController.swift
//  galaxy
//
//  Created by alexzzw on 2024/9/13.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
import SafariServices

class YHResignDocumentDetailViewController: YHBaseViewController {
    
    private var editEvent: (() -> Void)?
    private var submitEvent: (() -> Void)?
    private var signatureConfirmationEvent: (() -> Void)?
    private var previewEvent: (() -> Void)?
    private var infoConfirmationEvent: (() -> Void)?

    enum RowType {
        case tips(_ title: String, _ detail: NSAttributedString)
        case content(_ title: String, _ fileName: String, _ status: YHResignDocumentStatus, _ config: YHResignDocumentStatusCell.ButtonsConfig)
    }
    
    private var datas: [[RowType]] = []
    private lazy var viewModel = YHResignDocumentViewModel()
    
    private let docId: Int
    private var name: String
    
    init(_ docId: Int, _ name: String = "") {
        self.docId = docId
        self.name = name
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    private lazy var tableView: UITableView = {
        let view = UITableView(frame:.zero, style:.grouped)
        view.estimatedSectionHeaderHeight = 16.0
        view.estimatedSectionFooterHeight = 0.01
        view.sectionHeaderHeight = 16.0
        view.sectionFooterHeight = 0.01
        view.contentInsetAdjustmentBehavior = .never
        view.showsVerticalScrollIndicator = false
        view.separatorStyle = .none
        view.delegate = self
        view.dataSource = self
        view.backgroundColor = .clear
        view.tableFooterView = UITableViewHeaderFooterView()
        view.rowHeight = UITableView.automaticDimension
        view.register(YHResignDocumentTipsCell.self, forCellReuseIdentifier: YHResignDocumentTipsCell.cellReuseIdentifier)
        view.register(YHResignDocumentStatusCell.self, forCellReuseIdentifier: YHResignDocumentStatusCell.cellReuseIdentifier)
        return view
    }()

    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
        tableView.reloadData()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        requestData()
    }
}

extension YHResignDocumentDetailViewController {
    private func setupUI() {
        gk_navTitle = name
        gk_navBarAlpha = 1.0
        gk_navBackgroundColor = .white
        
        view.backgroundColor = UIColor.contentBkgColor
        view.addSubview(tableView)
        
        tableView.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalTo(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalToSuperview()
        }
    }
    
    private func requestData(isNeedLoading: Bool = false) {
        if isNeedLoading {
            YHHUD.show(.progress(message: "加载中..."))
        }
        viewModel.getDocumentDetail(docId: docId) { [weak self] documentModel, error in
            if isNeedLoading {
                YHHUD.hide()
            }
            guard let documentModel = documentModel else {
                printLog("YHResignDocumentDetailViewController:  请求失败")
                if let errorMsg = error?.errorMsg, errorMsg.count > 0 {
                    YHHUD.show(.error(message: errorMsg))
                    YHHUD.hide(afterDelay: 1)
                }
                return
            }
            guard let self = self else {
                return
            }
            var firstSetcion: [RowType] = []
            var secondSetcion: [RowType] = []
            if let docType = YHResignDocumentType(rawValue: documentModel.docType), let status = YHResignDocumentStatus.getStatus(documentModel.docStatus) {
                firstSetcion.append(.tips("​填写指引", docType.attributeInfo()))
                let config = self.getStatusButtonsConfig(docType: docType, status: status, model: documentModel)
                secondSetcion = [.content("文书稿件", self.name, status, config)]
                datas.removeAll()
                datas.append(firstSetcion)
                datas.append(secondSetcion)
                tableView.reloadData()
            }
        }
    }
}

extension YHResignDocumentDetailViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        datas.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        guard datas.count > section else {
            return 0
        }
        return datas[section].count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard datas.count > indexPath.section else {
            return UITableViewCell()
        }
        let sectionArr = datas[indexPath.section]
        guard sectionArr.count > indexPath.row else {
            return UITableViewCell()
        }
        let item = sectionArr[indexPath.row]
        
        switch item {
        case let .tips(title, detail):
            if let cell = tableView.dequeueReusableCell(withIdentifier: YHResignDocumentTipsCell.cellReuseIdentifier) as? YHResignDocumentTipsCell {
                cell.setupCellInfo(title: title, detail: detail)
                return cell
            }
        case let .content(title, fileName, status, config):
            if let cell = tableView.dequeueReusableCell(withIdentifier: YHResignDocumentStatusCell.cellReuseIdentifier) as? YHResignDocumentStatusCell {
                cell.setupCellInfo(title: title, fileName: fileName, status: status, buttonsConfig: config)
                return cell
            }
        }
        return UITableViewCell()
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        return UIView()
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        return UIView()
    }
}

extension YHResignDocumentDetailViewController {
    private func getStatusButtonsConfig(docType: YHResignDocumentType, status: YHResignDocumentStatus,  model: YHResignDocumentCompleteModel) -> YHResignDocumentStatusCell.ButtonsConfig {
        var config = YHResignDocumentStatusCell.ButtonsConfig()
        var actions: [(() -> Void)] = []
        editEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.getResginDocumentUrl(docId: model.docId, isEditable: true) { [weak self] model, error in
                guard let self = self else {
                    return
                }
                YHHUD.hide()
                if let urlString = model?.url, let url = URL(string: urlString) {
                    let safariViewController = SFSafariViewController(url: url)
                    safariViewController.dismissButtonStyle = .close
                    safariViewController.modalPresentationStyle = .fullScreen
                    self.present(safariViewController, animated: true, completion: nil)
                } else if let error = error {
                    let errorMsg = error.errorMsg
                    YHHUD.show(.error(message: errorMsg))
                    YHHUD.hide(afterDelay: 1)
                }
            }
            printLog("###editEvent")
        }
        
        submitEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.submitResginDocument(model.docId, nil) { success, error in
                YHHUD.hide()
                if success {
                    YHHUD.show(.success(message: "提交成功"))
                    YHHUD.hide(afterDelay: 1)
                    self.requestData(isNeedLoading: true)
                } else if let error = error {
                    let errorMsg = error.errorMsg
                    YHHUD.show(.error(message: errorMsg))
                    YHHUD.hide(afterDelay: 1)
                }
            }
            printLog("###submitEvent")
        }
        
        signatureConfirmationEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.resginDocumentSignature(model.docId, nil) { [weak self] signatureModel, error in
                YHHUD.hide()
                guard let self = self else {
                    return
                }
                YHHUD.hide()
                guard let signatureModel = signatureModel else {
                    if let error = error {
                        printLog("###续期文书签字: \(error.errorMsg)")
                        let errorMsg = error.errorMsg
                        YHHUD.show(.error(message: errorMsg))
                        YHHUD.hide(afterDelay: 1)
                    }
                    return
                }
                let url = signatureModel.url
                // 跳签字
                let vc = YHSignatureWebViewController()
                vc.titleName = self.name
                vc.realNameUrl = url
                self.navigationController?.pushViewController(vc)
            }
        }
        
        previewEvent = { [weak self] in
            guard let self = self else {
                return
            }
            YHHUD.show(.progress(message: "加载中..."))
            self.viewModel.getResginDocumentUrl(docId: model.docId, isEditable: false) { [weak self] model, error in
                guard let self = self else {
                    return
                }
                YHHUD.hide()
                if let urlString = model?.url, let url = URL(string: urlString) {
                    let safariViewController = SFSafariViewController(url: url)
                    safariViewController.dismissButtonStyle = .close
                    safariViewController.modalPresentationStyle = .fullScreen
                    self.present(safariViewController, animated: true, completion: nil)
                } else if let error = error {
                    let errorMsg = error.errorMsg
                    YHHUD.show(.error(message: errorMsg))
                    YHHUD.hide(afterDelay: 1)
                }
            }
            printLog("###previewEvent")
        }
        
        infoConfirmationEvent = { [weak self] in
            guard let self = self else {
                return
            }
//            let vc = YHH5WebViewVC()
//            vc.navTitle = self.name
//            // TODO: - Alex等链接下来再赋值
//            //vc.url = url
//            vc.isFullScreenFlag = false
//            self.navigationController?.pushViewController(vc)
//            printLog("###infoConfirmationEvent")
            
            
            printLog("跳转QMAS")
            if let param = YHLoginManager.shared.userModel?.token,param.count>5 {
                printLog(param)
                var token : String = param
                
                var url = YHBaseUrlManager.shared.curH5URL() + "superAppBridge.html#/guide/qmas"
                let addtionString = String.randomAlphaNumericStringT(6)//随机6个字母或者数字
                let indexToInsert = token.index(token.startIndex, offsetBy: 6) //插入位置，这里以第六个字符后面为例
                token.insert(contentsOf: addtionString, at: indexToInsert)
                //https://test-hkdiy-h5.galaxy-immi.com/superAppBridge.html#/guide/qmas?token=xx&doc_id=xx

                url = url + "?param=" + token + "&doc_id=\(model.docId)"
                let vc = YHH5WebViewVC()
                vc.navTitle = ""
                vc.url = url
                UIViewController.current?.navigationController?.pushViewController(vc)
            }
        }
        
        switch docType {
        case .powerOfAttorney, .settlementPlan, .explanatoryStatement:
            switch status {
            case .pendingConfirmation, .awaitingSignature:
                config.names = ["在线编辑", "提交", "签字确认"]
                if let editEvent = editEvent {
                    actions.append(editEvent)
                }
                if let submitEvent = submitEvent {
                    actions.append(submitEvent)
                }
                if let signatureConfirmationEvent = signatureConfirmationEvent {
                    actions.append(signatureConfirmationEvent)
                }
                config.actions = actions
                config.style = .three
            case .finalizing, .underReview, .completed:
                if let previewEvent = previewEvent {
                    actions.append(previewEvent)
                }
                config.names = ["查看"]
                config.actions = actions
                config.style = .one
            }
        case .qmasDoc:
            switch status {
            case .pendingConfirmation:
                if let infoConfirmationEvent = infoConfirmationEvent {
                    actions.append(infoConfirmationEvent)
                }
                config.names = ["信息确认"]
                config.actions = actions
                config.style = .one
            case .finalizing, .underReview, .completed:
                if let previewEvent = previewEvent {
                    actions.append(previewEvent)
                }
                config.names = ["查看"]
                config.actions = actions
                config.style = .one
            case .awaitingSignature:
                if let infoConfirmationEvent = infoConfirmationEvent {
                    actions.append(infoConfirmationEvent)
                }
                if let signatureConfirmationEvent = signatureConfirmationEvent {
                    actions.append(signatureConfirmationEvent)
                }
                config.names = ["信息确认", "签字确认"]
                config.actions = actions
                config.style = .two
            }
        }
        return config
    }
}

private extension YHResignDocumentType {
    func attributeInfo() -> NSAttributedString {
        switch self {
        case .powerOfAttorney:
            let normalText = "请主申请人完成电子签字请您尽快核对信息，如信息有误，可进行在线编辑如信息无误，"
            let attText = "请您在右下角的位置完成电子签字"
            let attStr = NSMutableAttributedString(string: normalText + attText)
            attStr.setAttributes([.foregroundColor: UIColor.brandMainColor], range: NSRange(location: normalText.count, length: attText.count))
            return attStr
        case .qmasDoc:
            let normalText = "请主申请人勾选信息并完成电子签字此文件需在申请续签时提交给港府请您尽快进行对应问题的勾选，填写完整邮箱、电话号码（内地电话号码或香港电话号码）、日期（填写当下日期即可）"
            let attText = "申请人姓名需要进行电子签字"
            let attStr = NSMutableAttributedString(string: normalText + attText)
            attStr.setAttributes([.foregroundColor: UIColor.brandMainColor], range: NSRange(location: normalText.count, length: attText.count))
            return attStr
        case .settlementPlan:
            let normalText = "请主申请人完成电子签字请您审阅赴港定居计划文书内容，如需调整和补充内容，请在线上进行编辑，我们将根据您的修改及反馈对文书进行调整如您对《定居计划书》无其他补充建议，"
            let attText = "请您在文书尾部进行电子签字"
            let attStr = NSMutableAttributedString(string: normalText + attText)
            attStr.setAttributes([.foregroundColor: UIColor.brandMainColor], range: NSRange(location: normalText.count, length: attText.count))
            return attStr
        case .explanatoryStatement:
            let normalText = "请主申请人完成电子签字请您审阅文书中的相关解释说明，如需调整和补充，请在线上进行编辑，我们将根据您的修改及反馈对文书进行调整如您对《解释说明书》无其他补充建议，"
            let attText = "请您在文书尾部进行电子签字"
            let attStr = NSMutableAttributedString(string: normalText + attText)
            attStr.setAttributes([.foregroundColor: UIColor.brandMainColor], range: NSRange(location: normalText.count, length: attText.count))
            return attStr
        }
    }
}
